"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_1 = require("vscode");
const util_1 = require("./util");
const keywords = [
    'theorem', 'lemma', 'axiom', 'axioms', 'variable', 'protected', 'private',
    'def', 'meta', 'mutual', 'example', 'noncomputable',
    'variables', 'parameter', 'parameters', 'constant', 'constants',
    'using_well_founded',
    'end', 'namespace', 'section', 'prelude',
    'import', 'inductive', 'coinductive', 'structure', 'class', 'universe', 'universes', 'local',
    'precedence', 'reserve', 'infixl', 'infixr', 'infix', 'postfix', 'prefix', 'notation',
    'set_option', 'open', 'export',
    'attribute', 'instance', 'include', 'omit',
    'declare_trace', 'add_key_equivalence',
    'run_cmd', '#check', '#reduce', '#eval', '#print', '#help', '#exit',
    '#compile', '#unify',
    'fun', 'Pi', 'let', 'in', 'at',
    'have', 'assume', 'show', 'suffices',
    'do', 'if', 'then', 'else', 'by',
    'hiding', 'replacing',
    'from',
    'Type', 'Sort',
    'with', 'without',
    'calc',
    'begin', 'using',
    'sorry',
    'match',
    'renaming', 'extends',
];
class LeanCompletionItemProvider {
    constructor(server) {
        this.server = server;
    }
    provideCompletionItems(document, position) {
        return __awaiter(this, void 0, void 0, function* () {
            // TODO(gabriel): use LeanInputAbbreviator.active() instead
            if (!util_1.isInputCompletion(document, position)) {
                const message = yield this.server.complete(document.fileName, position.line + 1, position.character);
                const completions = [];
                for (const completion of message.completions) {
                    const item = new vscode_1.CompletionItem(completion.text, vscode_1.CompletionItemKind.Function);
                    item.range = new vscode_1.Range(position.translate(0, -message.prefix.length), position);
                    if (completion.tactic_params) {
                        item.detail = completion.tactic_params.join(' ');
                    }
                    else {
                        item.detail = completion.type;
                    }
                    item.documentation = new vscode_1.MarkdownString(completion.doc);
                    completions.push(item);
                }
                for (const kw of keywords) {
                    completions.push(new vscode_1.CompletionItem(kw, vscode_1.CompletionItemKind.Keyword));
                }
                return completions;
            }
            else {
                return null;
            }
        });
    }
}
exports.LeanCompletionItemProvider = LeanCompletionItemProvider;
//# sourceMappingURL=completion.js.map