"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_1 = require("vscode");
function mkRange(r) {
    return new vscode_1.Range(r.start.line - 1, r.start.column, r.end.line - 1, r.end.column);
}
class LeanHoles {
    constructor(server, leanDocs) {
        this.server = server;
        this.leanDocs = leanDocs;
        this.subscriptions = [];
        this.executeHoleCommand = 'lean.executeHole';
        this.subscriptions.push(this.collection = vscode_1.languages.createDiagnosticCollection('lean holes'), vscode_1.commands.registerCommand(this.executeHoleCommand, (file, line, column, action) => this.execute(file, line, column, action)), vscode_1.languages.registerCodeActionsProvider(this.leanDocs, this), vscode_1.window.onDidChangeVisibleTextEditors(() => this.refresh()), this.server.statusChanged.on(() => this.refresh()));
    }
    refresh() {
        return __awaiter(this, void 0, void 0, function* () {
            const ress = yield Promise.all(vscode_1.window.visibleTextEditors
                .filter((editor) => vscode_1.languages.match(this.leanDocs, editor.document))
                .map((editor) => this.server.allHoleCommands(editor.document.fileName)));
            this.holes = [];
            for (const res of ress) {
                [].push.apply(this.holes, res.holes);
            }
            const holesPerFile = new Map();
            for (const hole of this.holes) {
                if (!holesPerFile.get(hole.file)) {
                    holesPerFile.set(hole.file, []);
                }
                holesPerFile.get(hole.file).push(hole);
            }
            this.collection.clear();
            for (const file of holesPerFile.keys()) {
                this.collection.set(vscode_1.Uri.file(file), holesPerFile.get(file).map((hole) => new vscode_1.Diagnostic(mkRange(hole), 'Hole: ' + hole.results.map((a) => a.name).join('/'), vscode_1.DiagnosticSeverity.Hint)));
            }
        });
    }
    execute(file, line, column, action) {
        return __awaiter(this, void 0, void 0, function* () {
            let res;
            try {
                res = yield this.server.hole(file, line, column, action);
            }
            catch (e) {
                return vscode_1.window.showErrorMessage(`Error while executing hole command: ${e}`);
            }
            if (res.message) {
                vscode_1.window.showInformationMessage(res.message);
            }
            if (res.replacements && res.replacements.alternatives) {
                // TODO(gabriel): ask user if more than one alternative
                for (const editor of vscode_1.window.visibleTextEditors) {
                    if (editor.document.fileName === file) {
                        editor.edit((builder) => {
                            builder.replace(mkRange(res.replacements), res.replacements.alternatives[0].code);
                        });
                    }
                }
            }
        });
    }
    provideCodeActions(document, range) {
        const cmds = [];
        for (const hole of this.holes) {
            if (!range.intersection(mkRange(hole))) {
                continue;
            }
            for (const action of hole.results) {
                cmds.push({
                    title: action.description,
                    command: this.executeHoleCommand,
                    arguments: [hole.file, hole.start.line, hole.start.column, action.name],
                });
            }
        }
        return cmds;
    }
    dispose() {
        for (const s of this.subscriptions) {
            s.dispose();
        }
    }
}
exports.LeanHoles = LeanHoles;
//# sourceMappingURL=holes.js.map