"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const vscode_1 = require("vscode");
class LeanpkgService {
    constructor(server) {
        this.server = server;
        this.subscriptions = [];
        this.leanpkgToml = path.join(vscode_1.workspace.rootPath, 'leanpkg.toml');
        this.leanpkgPath = path.join(vscode_1.workspace.rootPath, 'leanpkg.path');
        this.checkLeanpkgPathFile();
        this.checkLeanpkgPathContents();
        this.subscriptions.push(vscode_1.workspace.registerTaskProvider('leanpkg', this));
        const watcher = vscode_1.workspace.createFileSystemWatcher('**/leanpkg.*');
        watcher.onDidChange((u) => this.handleFileChanged(u));
        watcher.onDidCreate((u) => this.handleFileChanged(u));
        watcher.onDidDelete((u) => this.handleFileChanged(u));
        this.subscriptions.push(watcher);
    }
    dispose() {
        for (const s of this.subscriptions) {
            s.dispose();
        }
    }
    handleFileChanged(uri) {
        if (uri.fsPath === path.join(vscode_1.workspace.rootPath, 'leanpkg.toml')) {
            this.checkLeanpkgPathFile();
        }
        else if (uri.fsPath === path.join(vscode_1.workspace.rootPath, 'leanpkg.path')) {
            this.checkLeanpkgPathContents(true);
        }
    }
    mkTask(command) {
        const task = new vscode_1.Task({ type: 'leanpkg', command }, command, 'leanpkg', new vscode_1.ProcessExecution(this.leanpkgExecutable(), [command]), []);
        task.group = vscode_1.TaskGroup.Build;
        task.presentationOptions = {
            echo: true,
            focus: true,
        };
        return task;
    }
    provideTasks() {
        return ['build', 'configure', 'upgrade'].map((c) => this.mkTask(c));
    }
    resolveTask(task) {
        return undefined;
    }
    leanpkgExecutable() {
        const config = vscode_1.workspace.getConfiguration('lean');
        const leanpkg = config.get('leanpkgPath');
        if (leanpkg) {
            return leanpkg;
        }
        const leanPath = config.get('executablePath');
        if (leanPath) {
            const leanpkg2 = path.join(path.dirname(leanPath), 'lean');
            if (fs.existsSync(leanpkg2)) {
                return leanpkg2;
            }
        }
        return 'leanpkg';
    }
    checkLeanpkgPathContents(promptForRestart) {
        const oldContents = this.leanpkgPathContents;
        this.leanpkgPathContents = fs.existsSync(this.leanpkgPath) &&
            fs.readFileSync(this.leanpkgPath).toString();
        if (oldContents !== this.leanpkgPathContents && promptForRestart) {
            this.server.requestRestart('Lean: leanpkg.path changed.', true);
        }
    }
    checkLeanpkgPathFile() {
        if (!fs.existsSync(this.leanpkgToml)) {
            // TODO(gabriel): encourage users to use leanpkg
        }
        else if (!fs.existsSync(this.leanpkgPath)) {
            this.requestLeanpkgConfigure('Lean: leanpkg.path does not exist');
        }
        else if (fs.statSync(this.leanpkgPath) < fs.statSync(this.leanpkgToml)) {
            this.requestLeanpkgConfigure('Lean: leanpkg.path out of date');
        }
    }
    requestLeanpkgConfigure(message) {
        return __awaiter(this, void 0, void 0, function* () {
            const configureItem = 'Run leanpkg configure.';
            const chosen = yield vscode_1.window.showErrorMessage(message, configureItem);
            if (chosen === configureItem) {
                yield this.configure();
            }
        });
    }
    configure() {
        return __awaiter(this, void 0, void 0, function* () {
            yield vscode_1.commands.executeCommand('workbench.action.tasks.runTask', 'leanpkg: configure');
        });
    }
    build() {
        return __awaiter(this, void 0, void 0, function* () {
            yield vscode_1.commands.executeCommand('workbench.action.tasks.runTask', 'leanpkg: build');
        });
    }
    upgrade() {
        return __awaiter(this, void 0, void 0, function* () {
            yield vscode_1.commands.executeCommand('workbench.action.tasks.runTask', 'leanpkg: upgrade');
        });
    }
}
exports.LeanpkgService = LeanpkgService;
//# sourceMappingURL=leanpkg.js.map